# -*- coding: utf-8 -*-

"""
AWS Console URL full text search utility.
"""


from ._version import __version__

__short_description__ = "AWS Console URL full text search utility."
__license__ = "MIT"
__author__ = "Sanhe Hu"
__author_email__ = "husanhe@gmail.com"
__github_username__ = "MacHu-GWU"
